package net.minecraft.data.loot;

import java.util.function.BiConsumer;
import java.util.function.Consumer;
import net.minecraft.block.Blocks;
import net.minecraft.item.Items;
import net.minecraft.potion.Effects;
import net.minecraft.util.ResourceLocation;
import net.minecraft.world.storage.MapDecoration;
import net.minecraft.world.storage.loot.ConstantRange;
import net.minecraft.world.storage.loot.EmptyLootEntry;
import net.minecraft.world.storage.loot.ItemLootEntry;
import net.minecraft.world.storage.loot.LootPool;
import net.minecraft.world.storage.loot.LootTable;
import net.minecraft.world.storage.loot.LootTables;
import net.minecraft.world.storage.loot.RandomValueRange;
import net.minecraft.world.storage.loot.functions.EnchantRandomly;
import net.minecraft.world.storage.loot.functions.EnchantWithLevels;
import net.minecraft.world.storage.loot.functions.ExplorationMap;
import net.minecraft.world.storage.loot.functions.SetCount;
import net.minecraft.world.storage.loot.functions.SetStewEffect;

public class ChestLootTables implements Consumer<BiConsumer<ResourceLocation, LootTable.Builder>> {
   public void accept(BiConsumer<ResourceLocation, LootTable.Builder> p_accept_1_) {
      p_accept_1_.accept(LootTables.CHESTS_ABANDONED_MINESHAFT, LootTable.builder().addLootPool(LootPool.builder().rolls(ConstantRange.of(1)).addEntry(ItemLootEntry.builder(Items.GOLDEN_APPLE).weight(20)).addEntry(ItemLootEntry.builder(Items.ENCHANTED_GOLDEN_APPLE)).addEntry(ItemLootEntry.builder(Items.NAME_TAG).weight(30)).addEntry(ItemLootEntry.builder(Items.BOOK).weight(10).acceptFunction(EnchantRandomly.func_215900_c())).addEntry(ItemLootEntry.builder(Items.IRON_PICKAXE).weight(5)).addEntry(EmptyLootEntry.func_216167_a().weight(5))).addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 4.0F)).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.REDSTONE).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(4.0F, 9.0F)))).addEntry(ItemLootEntry.builder(Items.LAPIS_LAZULI).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(4.0F, 9.0F)))).addEntry(ItemLootEntry.builder(Items.DIAMOND).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 2.0F)))).addEntry(ItemLootEntry.builder(Items.COAL).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(3.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.MELON_SEEDS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.PUMPKIN_SEEDS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.BEETROOT_SEEDS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 4.0F))))).addLootPool(LootPool.builder().rolls(ConstantRange.of(3)).addEntry(ItemLootEntry.builder(Blocks.RAIL).weight(20).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(4.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Blocks.POWERED_RAIL).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Blocks.DETECTOR_RAIL).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Blocks.ACTIVATOR_RAIL).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Blocks.TORCH).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 16.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_BURIED_TREASURE, LootTable.builder().addLootPool(LootPool.builder().rolls(ConstantRange.of(1)).addEntry(ItemLootEntry.builder(Items.HEART_OF_THE_SEA))).addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(5.0F, 8.0F)).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(20).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Blocks.TNT).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 2.0F))))).addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 3.0F)).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(4.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.DIAMOND).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 2.0F)))).addEntry(ItemLootEntry.builder(Items.PRISMARINE_CRYSTALS).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F))))).addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(0.0F, 1.0F)).addEntry(ItemLootEntry.builder(Items.LEATHER_CHESTPLATE)).addEntry(ItemLootEntry.builder(Items.IRON_SWORD))).addLootPool(LootPool.builder().rolls(ConstantRange.of(2)).addEntry(ItemLootEntry.builder(Items.COOKED_COD).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.COOKED_SALMON).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 4.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_DESERT_PYRAMID, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 4.0F)).addEntry(ItemLootEntry.builder(Items.DIAMOND).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.BONE).weight(25).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(4.0F, 6.0F)))).addEntry(ItemLootEntry.builder(Items.SPIDER_EYE).weight(25).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.ROTTEN_FLESH).weight(25).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(3.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.SADDLE).weight(20)).addEntry(ItemLootEntry.builder(Items.IRON_HORSE_ARMOR).weight(15)).addEntry(ItemLootEntry.builder(Items.GOLDEN_HORSE_ARMOR).weight(10)).addEntry(ItemLootEntry.builder(Items.DIAMOND_HORSE_ARMOR).weight(5)).addEntry(ItemLootEntry.builder(Items.BOOK).weight(20).acceptFunction(EnchantRandomly.func_215900_c())).addEntry(ItemLootEntry.builder(Items.GOLDEN_APPLE).weight(20)).addEntry(ItemLootEntry.builder(Items.ENCHANTED_GOLDEN_APPLE).weight(2)).addEntry(EmptyLootEntry.func_216167_a().weight(15))).addLootPool(LootPool.builder().rolls(ConstantRange.of(4)).addEntry(ItemLootEntry.builder(Items.BONE).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.GUNPOWDER).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.ROTTEN_FLESH).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.STRING).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Blocks.SAND).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_END_CITY_TREASURE, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 6.0F)).addEntry(ItemLootEntry.builder(Items.DIAMOND).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(4.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 6.0F)))).addEntry(ItemLootEntry.builder(Items.BEETROOT_SEEDS).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 10.0F)))).addEntry(ItemLootEntry.builder(Items.SADDLE).weight(3)).addEntry(ItemLootEntry.builder(Items.IRON_HORSE_ARMOR)).addEntry(ItemLootEntry.builder(Items.GOLDEN_HORSE_ARMOR)).addEntry(ItemLootEntry.builder(Items.DIAMOND_HORSE_ARMOR)).addEntry(ItemLootEntry.builder(Items.DIAMOND_SWORD).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.DIAMOND_BOOTS).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.DIAMOND_CHESTPLATE).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.DIAMOND_LEGGINGS).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.DIAMOND_HELMET).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.DIAMOND_PICKAXE).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.DIAMOND_SHOVEL).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.IRON_SWORD).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.IRON_BOOTS).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.IRON_CHESTPLATE).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.IRON_LEGGINGS).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.IRON_HELMET).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.IRON_PICKAXE).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e())).addEntry(ItemLootEntry.builder(Items.IRON_SHOVEL).weight(3).acceptFunction(EnchantWithLevels.func_215895_a(RandomValueRange.func_215837_a(20.0F, 39.0F)).func_216059_e()))));
      p_accept_1_.accept(LootTables.CHESTS_IGLOO_CHEST, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 8.0F)).addEntry(ItemLootEntry.builder(Items.APPLE).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.COAL).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_NUGGET).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.STONE_AXE).weight(2)).addEntry(ItemLootEntry.builder(Items.ROTTEN_FLESH).weight(10)).addEntry(ItemLootEntry.builder(Items.EMERALD)).addEntry(ItemLootEntry.builder(Items.WHEAT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 3.0F))))).addLootPool(LootPool.builder().rolls(ConstantRange.of(1)).addEntry(ItemLootEntry.builder(Items.GOLDEN_APPLE))));
      p_accept_1_.accept(LootTables.CHESTS_JUNGLE_TEMPLE, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 6.0F)).addEntry(ItemLootEntry.builder(Items.DIAMOND).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Blocks.BAMBOO).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.BONE).weight(20).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(4.0F, 6.0F)))).addEntry(ItemLootEntry.builder(Items.ROTTEN_FLESH).weight(16).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(3.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.SADDLE).weight(3)).addEntry(ItemLootEntry.builder(Items.IRON_HORSE_ARMOR)).addEntry(ItemLootEntry.builder(Items.GOLDEN_HORSE_ARMOR)).addEntry(ItemLootEntry.builder(Items.DIAMOND_HORSE_ARMOR)).addEntry(ItemLootEntry.builder(Items.BOOK).acceptFunction(EnchantWithLevels.func_215895_a(ConstantRange.of(30)).func_216059_e()))));
      p_accept_1_.accept(LootTables.CHESTS_JUNGLE_TEMPLE_DISPENSER, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 2.0F)).addEntry(ItemLootEntry.builder(Items.ARROW).weight(30).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 7.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_NETHER_BRIDGE, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 4.0F)).addEntry(ItemLootEntry.builder(Items.DIAMOND).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.GOLDEN_SWORD).weight(5)).addEntry(ItemLootEntry.builder(Items.GOLDEN_CHESTPLATE).weight(5)).addEntry(ItemLootEntry.builder(Items.FLINT_AND_STEEL).weight(5)).addEntry(ItemLootEntry.builder(Items.NETHER_WART).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(3.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.SADDLE).weight(10)).addEntry(ItemLootEntry.builder(Items.GOLDEN_HORSE_ARMOR).weight(8)).addEntry(ItemLootEntry.builder(Items.IRON_HORSE_ARMOR).weight(5)).addEntry(ItemLootEntry.builder(Items.DIAMOND_HORSE_ARMOR).weight(3)).addEntry(ItemLootEntry.builder(Blocks.OBSIDIAN).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 4.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_PILLAGER_OUTPOST, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(0.0F, 1.0F)).addEntry(ItemLootEntry.builder(Items.CROSSBOW))).addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 3.0F)).addEntry(ItemLootEntry.builder(Items.WHEAT).weight(7).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(3.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.POTATO).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.CARROT).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(3.0F, 5.0F))))).addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 3.0F)).addEntry(ItemLootEntry.builder(Blocks.DARK_OAK_LOG).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 3.0F))))).addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 3.0F)).addEntry(ItemLootEntry.builder(Items.EXPERIENCE_BOTTLE).weight(7)).addEntry(ItemLootEntry.builder(Items.STRING).weight(4).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 6.0F)))).addEntry(ItemLootEntry.builder(Items.ARROW).weight(4).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.TRIPWIRE_HOOK).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.BOOK).weight(1).acceptFunction(EnchantRandomly.func_215900_c()))));
      p_accept_1_.accept(LootTables.CHESTS_SHIPWRECK_MAP, LootTable.builder().addLootPool(LootPool.builder().rolls(ConstantRange.of(1)).addEntry(ItemLootEntry.builder(Items.MAP).acceptFunction(ExplorationMap.func_215903_b().func_216065_a("buried_treasure").func_216064_a(MapDecoration.Type.RED_X).func_216062_a((byte)1).func_216063_a(false)))).addLootPool(LootPool.builder().rolls(ConstantRange.of(3)).addEntry(ItemLootEntry.builder(Items.COMPASS)).addEntry(ItemLootEntry.builder(Items.MAP)).addEntry(ItemLootEntry.builder(Items.CLOCK)).addEntry(ItemLootEntry.builder(Items.PAPER).weight(20).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 10.0F)))).addEntry(ItemLootEntry.builder(Items.FEATHER).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.BOOK).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_SHIPWRECK_SUPPLY, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(3.0F, 10.0F)).addEntry(ItemLootEntry.builder(Items.PAPER).weight(8).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 12.0F)))).addEntry(ItemLootEntry.builder(Items.POTATO).weight(7).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 6.0F)))).addEntry(ItemLootEntry.builder(Items.POISONOUS_POTATO).weight(7).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 6.0F)))).addEntry(ItemLootEntry.builder(Items.CARROT).weight(7).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(4.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.WHEAT).weight(7).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(8.0F, 21.0F)))).addEntry(ItemLootEntry.builder(Items.SUSPICIOUS_STEW).weight(10).acceptFunction(SetStewEffect.func_215948_b().func_216077_a(Effects.NIGHT_VISION, RandomValueRange.func_215837_a(7.0F, 10.0F)).func_216077_a(Effects.JUMP_BOOST, RandomValueRange.func_215837_a(7.0F, 10.0F)).func_216077_a(Effects.WEAKNESS, RandomValueRange.func_215837_a(6.0F, 8.0F)).func_216077_a(Effects.BLINDNESS, RandomValueRange.func_215837_a(5.0F, 7.0F)).func_216077_a(Effects.POISON, RandomValueRange.func_215837_a(10.0F, 20.0F)).func_216077_a(Effects.SATURATION, RandomValueRange.func_215837_a(7.0F, 10.0F)))).addEntry(ItemLootEntry.builder(Items.COAL).weight(6).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.ROTTEN_FLESH).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(5.0F, 24.0F)))).addEntry(ItemLootEntry.builder(Blocks.PUMPKIN).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Blocks.BAMBOO).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.GUNPOWDER).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Blocks.TNT).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 2.0F)))).addEntry(ItemLootEntry.builder(Items.LEATHER_HELMET).weight(3).acceptFunction(EnchantRandomly.func_215900_c())).addEntry(ItemLootEntry.builder(Items.LEATHER_CHESTPLATE).weight(3).acceptFunction(EnchantRandomly.func_215900_c())).addEntry(ItemLootEntry.builder(Items.LEATHER_LEGGINGS).weight(3).acceptFunction(EnchantRandomly.func_215900_c())).addEntry(ItemLootEntry.builder(Items.LEATHER_BOOTS).weight(3).acceptFunction(EnchantRandomly.func_215900_c()))));
      p_accept_1_.accept(LootTables.CHESTS_SHIPWRECK_TREASURE, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(3.0F, 6.0F)).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(90).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(40).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.DIAMOND).weight(5)).addEntry(ItemLootEntry.builder(Items.EXPERIENCE_BOTTLE).weight(5))).addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 5.0F)).addEntry(ItemLootEntry.builder(Items.IRON_NUGGET).weight(50).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 10.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_NUGGET).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 10.0F)))).addEntry(ItemLootEntry.builder(Items.LAPIS_LAZULI).weight(20).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 10.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_SIMPLE_DUNGEON, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 3.0F)).addEntry(ItemLootEntry.builder(Items.SADDLE).weight(20)).addEntry(ItemLootEntry.builder(Items.GOLDEN_APPLE).weight(15)).addEntry(ItemLootEntry.builder(Items.ENCHANTED_GOLDEN_APPLE).weight(2)).addEntry(ItemLootEntry.builder(Items.MUSIC_DISC_13).weight(15)).addEntry(ItemLootEntry.builder(Items.MUSIC_DISC_CAT).weight(15)).addEntry(ItemLootEntry.builder(Items.NAME_TAG).weight(20)).addEntry(ItemLootEntry.builder(Items.GOLDEN_HORSE_ARMOR).weight(10)).addEntry(ItemLootEntry.builder(Items.IRON_HORSE_ARMOR).weight(15)).addEntry(ItemLootEntry.builder(Items.DIAMOND_HORSE_ARMOR).weight(5)).addEntry(ItemLootEntry.builder(Items.BOOK).weight(10).acceptFunction(EnchantRandomly.func_215900_c()))).addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 4.0F)).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(20)).addEntry(ItemLootEntry.builder(Items.WHEAT).weight(20).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.BUCKET).weight(10)).addEntry(ItemLootEntry.builder(Items.REDSTONE).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.COAL).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.MELON_SEEDS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.PUMPKIN_SEEDS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.BEETROOT_SEEDS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 4.0F))))).addLootPool(LootPool.builder().rolls(ConstantRange.of(3)).addEntry(ItemLootEntry.builder(Items.BONE).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.GUNPOWDER).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.ROTTEN_FLESH).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.STRING).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_SPAWN_BONUS_CHEST, LootTable.builder().addLootPool(LootPool.builder().rolls(ConstantRange.of(1)).addEntry(ItemLootEntry.builder(Items.STONE_AXE)).addEntry(ItemLootEntry.builder(Items.WOODEN_AXE).weight(3))).addLootPool(LootPool.builder().rolls(ConstantRange.of(1)).addEntry(ItemLootEntry.builder(Items.STONE_PICKAXE)).addEntry(ItemLootEntry.builder(Items.WOODEN_PICKAXE).weight(3))).addLootPool(LootPool.builder().rolls(ConstantRange.of(3)).addEntry(ItemLootEntry.builder(Items.APPLE).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 2.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 2.0F)))).addEntry(ItemLootEntry.builder(Items.SALMON).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 2.0F))))).addLootPool(LootPool.builder().rolls(ConstantRange.of(4)).addEntry(ItemLootEntry.builder(Items.STICK).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 12.0F)))).addEntry(ItemLootEntry.builder(Blocks.OAK_PLANKS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 12.0F)))).addEntry(ItemLootEntry.builder(Blocks.OAK_LOG).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Blocks.SPRUCE_LOG).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Blocks.BIRCH_LOG).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Blocks.JUNGLE_LOG).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Blocks.ACACIA_LOG).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Blocks.DARK_OAK_LOG).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_STRONGHOLD_CORRIDOR, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 3.0F)).addEntry(ItemLootEntry.builder(Items.ENDER_PEARL).weight(10)).addEntry(ItemLootEntry.builder(Items.DIAMOND).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.REDSTONE).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(4.0F, 9.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.APPLE).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_PICKAXE).weight(5)).addEntry(ItemLootEntry.builder(Items.IRON_SWORD).weight(5)).addEntry(ItemLootEntry.builder(Items.IRON_CHESTPLATE).weight(5)).addEntry(ItemLootEntry.builder(Items.IRON_HELMET).weight(5)).addEntry(ItemLootEntry.builder(Items.IRON_LEGGINGS).weight(5)).addEntry(ItemLootEntry.builder(Items.IRON_BOOTS).weight(5)).addEntry(ItemLootEntry.builder(Items.GOLDEN_APPLE)).addEntry(ItemLootEntry.builder(Items.SADDLE)).addEntry(ItemLootEntry.builder(Items.IRON_HORSE_ARMOR)).addEntry(ItemLootEntry.builder(Items.GOLDEN_HORSE_ARMOR)).addEntry(ItemLootEntry.builder(Items.DIAMOND_HORSE_ARMOR)).addEntry(ItemLootEntry.builder(Items.BOOK).acceptFunction(EnchantWithLevels.func_215895_a(ConstantRange.of(30)).func_216059_e()))));
      p_accept_1_.accept(LootTables.CHESTS_STRONGHOLD_CROSSING, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 4.0F)).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.REDSTONE).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(4.0F, 9.0F)))).addEntry(ItemLootEntry.builder(Items.COAL).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(3.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.APPLE).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_PICKAXE)).addEntry(ItemLootEntry.builder(Items.BOOK).acceptFunction(EnchantWithLevels.func_215895_a(ConstantRange.of(30)).func_216059_e()))));
      p_accept_1_.accept(LootTables.CHESTS_STRONGHOLD_LIBRARY, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 10.0F)).addEntry(ItemLootEntry.builder(Items.BOOK).weight(20).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.PAPER).weight(20).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.MAP)).addEntry(ItemLootEntry.builder(Items.COMPASS)).addEntry(ItemLootEntry.builder(Items.BOOK).weight(10).acceptFunction(EnchantWithLevels.func_215895_a(ConstantRange.of(30)).func_216059_e()))));
      p_accept_1_.accept(LootTables.CHESTS_UNDERWATER_RUIN_BIG, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 8.0F)).addEntry(ItemLootEntry.builder(Items.COAL).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_NUGGET).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.EMERALD)).addEntry(ItemLootEntry.builder(Items.WHEAT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 3.0F))))).addLootPool(LootPool.builder().rolls(ConstantRange.of(1)).addEntry(ItemLootEntry.builder(Items.GOLDEN_APPLE)).addEntry(ItemLootEntry.builder(Items.BOOK).weight(5).acceptFunction(EnchantRandomly.func_215900_c())).addEntry(ItemLootEntry.builder(Items.LEATHER_CHESTPLATE)).addEntry(ItemLootEntry.builder(Items.GOLDEN_HELMET)).addEntry(ItemLootEntry.builder(Items.FISHING_ROD).weight(5).acceptFunction(EnchantRandomly.func_215900_c())).addEntry(ItemLootEntry.builder(Items.MAP).weight(10).acceptFunction(ExplorationMap.func_215903_b().func_216065_a("buried_treasure").func_216064_a(MapDecoration.Type.RED_X).func_216062_a((byte)1).func_216063_a(false)))));
      p_accept_1_.accept(LootTables.CHESTS_UNDERWATER_RUIN_SMALL, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(2.0F, 8.0F)).addEntry(ItemLootEntry.builder(Items.COAL).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.STONE_AXE).weight(2)).addEntry(ItemLootEntry.builder(Items.ROTTEN_FLESH).weight(5)).addEntry(ItemLootEntry.builder(Items.EMERALD)).addEntry(ItemLootEntry.builder(Items.WHEAT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 3.0F))))).addLootPool(LootPool.builder().rolls(ConstantRange.of(1)).addEntry(ItemLootEntry.builder(Items.LEATHER_CHESTPLATE)).addEntry(ItemLootEntry.builder(Items.GOLDEN_HELMET)).addEntry(ItemLootEntry.builder(Items.FISHING_ROD).weight(5).acceptFunction(EnchantRandomly.func_215900_c())).addEntry(ItemLootEntry.builder(Items.MAP).weight(5).acceptFunction(ExplorationMap.func_215903_b().func_216065_a("buried_treasure").func_216064_a(MapDecoration.Type.RED_X).func_216062_a((byte)1).func_216063_a(false)))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_WEAPONSMITH, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(3.0F, 8.0F)).addEntry(ItemLootEntry.builder(Items.DIAMOND).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.APPLE).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_PICKAXE).weight(5)).addEntry(ItemLootEntry.builder(Items.IRON_SWORD).weight(5)).addEntry(ItemLootEntry.builder(Items.IRON_CHESTPLATE).weight(5)).addEntry(ItemLootEntry.builder(Items.IRON_HELMET).weight(5)).addEntry(ItemLootEntry.builder(Items.IRON_LEGGINGS).weight(5)).addEntry(ItemLootEntry.builder(Items.IRON_BOOTS).weight(5)).addEntry(ItemLootEntry.builder(Blocks.OBSIDIAN).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(3.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Blocks.OAK_SAPLING).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(3.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.SADDLE).weight(3)).addEntry(ItemLootEntry.builder(Items.IRON_HORSE_ARMOR)).addEntry(ItemLootEntry.builder(Items.GOLDEN_HORSE_ARMOR)).addEntry(ItemLootEntry.builder(Items.DIAMOND_HORSE_ARMOR))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_TOOLSMITH, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(3.0F, 8.0F)).addEntry(ItemLootEntry.builder(Items.DIAMOND).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_PICKAXE).weight(5)).addEntry(ItemLootEntry.builder(Items.COAL).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.STICK).weight(20).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_SHOVEL).weight(5))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_CARTOGRAPHER, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 5.0F)).addEntry(ItemLootEntry.builder(Items.MAP).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.PAPER).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.COMPASS).weight(5)).addEntry(ItemLootEntry.builder(Items.BREAD).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.STICK).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 2.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_MASON, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 5.0F)).addEntry(ItemLootEntry.builder(Items.CLAY_BALL).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.FLOWER_POT).weight(1)).addEntry(ItemLootEntry.builder(Blocks.STONE).weight(2)).addEntry(ItemLootEntry.builder(Blocks.STONE_BRICKS).weight(2)).addEntry(ItemLootEntry.builder(Items.BREAD).weight(4).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.YELLOW_DYE).weight(1)).addEntry(ItemLootEntry.builder(Blocks.SMOOTH_STONE).weight(1)).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(1))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_ARMORER, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 5.0F)).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(4).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.IRON_HELMET).weight(1)).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(1))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_SHEPHERD, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 5.0F)).addEntry(ItemLootEntry.builder(Blocks.WHITE_WOOL).weight(6).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Blocks.BLACK_WOOL).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Blocks.GRAY_WOOL).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Blocks.BROWN_WOOL).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Blocks.LIGHT_GRAY_WOOL).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(1)).addEntry(ItemLootEntry.builder(Items.SHEARS).weight(1)).addEntry(ItemLootEntry.builder(Items.WHEAT).weight(6).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 6.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_BUTCHER, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 5.0F)).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(1)).addEntry(ItemLootEntry.builder(Items.PORKCHOP).weight(6).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.WHEAT).weight(6).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.BEEF).weight(6).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.MUTTON).weight(6).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.COAL).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_FLETCHER, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 5.0F)).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(1)).addEntry(ItemLootEntry.builder(Items.ARROW).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.FEATHER).weight(6).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.EGG).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.FLINT).weight(6).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.STICK).weight(6).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_FISHER, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 5.0F)).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(1)).addEntry(ItemLootEntry.builder(Items.COD).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.SALMON).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.WATER_BUCKET).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.BARREL).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.WHEAT_SEEDS).weight(3).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.COAL).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_TANNERY, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 5.0F)).addEntry(ItemLootEntry.builder(Items.LEATHER).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.LEATHER_CHESTPLATE).weight(2)).addEntry(ItemLootEntry.builder(Items.LEATHER_BOOTS).weight(2)).addEntry(ItemLootEntry.builder(Items.LEATHER_HELMET).weight(2)).addEntry(ItemLootEntry.builder(Items.BREAD).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.LEATHER_LEGGINGS).weight(2)).addEntry(ItemLootEntry.builder(Items.SADDLE).weight(1)).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_TEMPLE, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(3.0F, 8.0F)).addEntry(ItemLootEntry.builder(Items.REDSTONE).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(7).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.ROTTEN_FLESH).weight(7).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.LAPIS_LAZULI).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_PLAINS_HOUSE, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(3.0F, 8.0F)).addEntry(ItemLootEntry.builder(Items.GOLD_NUGGET).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.DANDELION).weight(2)).addEntry(ItemLootEntry.builder(Items.POPPY).weight(1)).addEntry(ItemLootEntry.builder(Items.POTATO).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.APPLE).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.BOOK).weight(1)).addEntry(ItemLootEntry.builder(Items.FEATHER).weight(1)).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Blocks.OAK_SAPLING).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 2.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_TAIGA_HOUSE, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(3.0F, 8.0F)).addEntry(ItemLootEntry.builder(Items.IRON_NUGGET).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.FERN).weight(2)).addEntry(ItemLootEntry.builder(Items.LARGE_FERN).weight(2)).addEntry(ItemLootEntry.builder(Items.POTATO).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.SWEET_BERRIES).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.PUMPKIN_SEEDS).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.PUMPKIN_PIE).weight(1)).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Blocks.SPRUCE_SAPLING).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.SPRUCE_SIGN).weight(1)).addEntry(ItemLootEntry.builder(Items.SPRUCE_LOG).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_SAVANNA_HOUSE, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(3.0F, 8.0F)).addEntry(ItemLootEntry.builder(Items.GOLD_NUGGET).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.GRASS).weight(5)).addEntry(ItemLootEntry.builder(Items.TALL_GRASS).weight(5)).addEntry(ItemLootEntry.builder(Items.BREAD).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.WHEAT_SEEDS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Blocks.ACACIA_SAPLING).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 2.0F)))).addEntry(ItemLootEntry.builder(Items.SADDLE).weight(1)).addEntry(ItemLootEntry.builder(Blocks.TORCH).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 2.0F)))).addEntry(ItemLootEntry.builder(Items.BUCKET).weight(1))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_SNOWY_HOUSE, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(3.0F, 8.0F)).addEntry(ItemLootEntry.builder(Blocks.BLUE_ICE).weight(1)).addEntry(ItemLootEntry.builder(Blocks.SNOW_BLOCK).weight(4)).addEntry(ItemLootEntry.builder(Items.POTATO).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.BEETROOT_SEEDS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 5.0F)))).addEntry(ItemLootEntry.builder(Items.BEETROOT_SOUP).weight(1)).addEntry(ItemLootEntry.builder(Items.FURNACE).weight(1)).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.SNOWBALL).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.COAL).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_VILLAGE_VILLAGE_DESERT_HOUSE, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(3.0F, 8.0F)).addEntry(ItemLootEntry.builder(Items.CLAY_BALL).weight(1)).addEntry(ItemLootEntry.builder(Items.GREEN_DYE).weight(1)).addEntry(ItemLootEntry.builder(Blocks.CACTUS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.WHEAT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 7.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.BOOK).weight(1)).addEntry(ItemLootEntry.builder(Blocks.DEAD_BUSH).weight(2).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F)))).addEntry(ItemLootEntry.builder(Items.EMERALD).weight(1).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 3.0F))))));
      p_accept_1_.accept(LootTables.CHESTS_WOODLAND_MANSION, LootTable.builder().addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 3.0F)).addEntry(ItemLootEntry.builder(Items.LEAD).weight(20)).addEntry(ItemLootEntry.builder(Items.GOLDEN_APPLE).weight(15)).addEntry(ItemLootEntry.builder(Items.ENCHANTED_GOLDEN_APPLE).weight(2)).addEntry(ItemLootEntry.builder(Items.MUSIC_DISC_13).weight(15)).addEntry(ItemLootEntry.builder(Items.MUSIC_DISC_CAT).weight(15)).addEntry(ItemLootEntry.builder(Items.NAME_TAG).weight(20)).addEntry(ItemLootEntry.builder(Items.CHAINMAIL_CHESTPLATE).weight(10)).addEntry(ItemLootEntry.builder(Items.DIAMOND_HOE).weight(15)).addEntry(ItemLootEntry.builder(Items.DIAMOND_CHESTPLATE).weight(5)).addEntry(ItemLootEntry.builder(Items.BOOK).weight(10).acceptFunction(EnchantRandomly.func_215900_c()))).addLootPool(LootPool.builder().rolls(RandomValueRange.func_215837_a(1.0F, 4.0F)).addEntry(ItemLootEntry.builder(Items.IRON_INGOT).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.GOLD_INGOT).weight(5).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.BREAD).weight(20)).addEntry(ItemLootEntry.builder(Items.WHEAT).weight(20).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.BUCKET).weight(10)).addEntry(ItemLootEntry.builder(Items.REDSTONE).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.COAL).weight(15).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.MELON_SEEDS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.PUMPKIN_SEEDS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 4.0F)))).addEntry(ItemLootEntry.builder(Items.BEETROOT_SEEDS).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(2.0F, 4.0F))))).addLootPool(LootPool.builder().rolls(ConstantRange.of(3)).addEntry(ItemLootEntry.builder(Items.BONE).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.GUNPOWDER).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.ROTTEN_FLESH).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F)))).addEntry(ItemLootEntry.builder(Items.STRING).weight(10).acceptFunction(SetCount.func_215932_a(RandomValueRange.func_215837_a(1.0F, 8.0F))))));
   }
}
